from lumo.contrib.itertools import poll
from lumo import DatasetBuilder, DataModule, DataBundler

x = DatasetBuilder().add_input('x', range(100)).add_output('x', 'x').DataLoader()
y = DatasetBuilder().add_input('x', range(200, 300)).add_output('x', 'x').DataLoader()

bundler = DataBundler().add(x).add(y).poll_mode()
# for i in bundler:
#     print(i)

print(len(bundler))



import os
from torch import distributed as dist


def local_rank() -> int:
    """
    A safe function that always returns a number to identify the rank of current process group.

    Rank is a unique identifier assigned to each process within a distributed
    process group. They are always consecutive integers ranging from 0 to
    ``world_size``.

    Returns:
        The rank of the process group
        -1, if not part of the group

    """
    rank = os.environ.get('LOCAL_RANK', -1)
    if rank == -1:
        if dist.is_available() and dist.is_initialized():
            rank = dist.get_rank()

    return int(rank)


def world_size() -> int:
    """
    A safe function that always returns the number of processes in the current process group

    Returns:
        The world size of the process group
        -1, if not part of the group
    Returns:

    """
    size = os.environ.get('WORLD_SIZE', 0)
    if size == 0:
        if dist.is_available() and dist.is_initialized():
            size = dist.get_world_size()
    return int(size)


def is_dist() -> bool:
    """
    Returns `True` if current program is in distribute mode, or `False` if not.
    """
    return local_rank() >= 0



"""
Methods about files/paths
"""
from lumo.utils import re
import os


def checkpath(*path, is_file=False):
    """

    Args:
        *path:

    Returns:

    """
    res = os.path.join(*path)
    if is_file:
        os.makedirs(os.path.dirname(res), exist_ok=True)
    else:
        os.makedirs(res, exist_ok=True)
    if not os.path.exists(res):
        return None
    return res


def compare_path(a, b):
    """
    Returns `True` if two path is equal, else `False`.
    """
    if a is None and b is None:
        return True
    if a is None or b is None:
        return False

    a, b = os.path.realpath(a), os.path.realpath(b)
    return os.path.normpath(a) == os.path.normpath(b)


def walk_files(root, suf='.ft'):
    """
    Recursively yields all files ends with a given suffix under the `root` directory.
    File will be reformatted in absolute path.

    Args:
        root: iterate directory path.
        suf:

    Returns:

    """
    if suf != '':
        suf = f'{suf.lstrip(".")}'  # check suffix
    for root, dirs, fs in os.walk(root):
        for f in sorted(fs):
            if f.endswith(suf):
                yield os.path.join(root, f)


def filter_filename(title: str, substr='-'):
    """Create a valid path by replacing invalid character into the given `substr`."""
    title = re.sub('[\/:*?"<>|]', substr, title)
    return title
